const assert = require('assert');
const { Before, Given, When, Then } = require('@cucumber/cucumber');

const axios = require("axios");
const MockAdapter = require("axios-mock-adapter");

// This sets the mock adapter on the default instance
const mockAxios = new MockAdapter(axios);

const { fixtures } = require('../../test/fixtures/server');

const mockValidConfig = fixtures.get('valid-config');
const mockInvalidConfig = fixtures.get('invalid-config');

const mockSchema = require('../../test/mock/swagger.json');
const { Adapter } = require('../../dist/adapter');
const { GenericProtocol } = require('../../dist/protocol');

Before(function() {

  mockAxios.onGet(
    `${mockValidConfig.api}/token`, 
    { params: { client_id: mockValidConfig.client_id, secret: mockValidConfig.secret } }
  ).reply(200, fixtures.get('response-valid-token'));

  mockAxios.onGet(
    `${mockInvalidConfig.api}/token`, 
    { params: { client_id: mockInvalidConfig.client_id, secret: mockInvalidConfig.secret } }
  ).reply(403, fixtures.get('response-denied-token'));

});

Given('valid config', function() {
  this.schema = mockSchema;
  this.config = mockValidConfig
});

Then('the adapter credentials are populated', function() {
  let mockProtocol = new GenericProtocol(this.schema);
  let mockAdapter = new Adapter(mockProtocol, this.config);
  this.adapter = mockAdapter; 
  assert.equal(this.adapter.credentials.token, fixtures.get('response-valid-token').token);
});

Given('invalid config', function() {
  this.schema = mockSchema;
  this.config = mockInvalidConfig;
});

Then('the adapter auth fails', function() {
  assert.throws(
    function() {
      let mockProtocol = new GenericProtocol(mockSchema);
      let mockAdapter = new Adapter(mockProtocol, mockInvalidConfig);
      this.adapter = mockAdapter;     
    },
    {
      name: "AxiosError"
    }
  );
});